/*
 * OperatorSupport.java
 * Created on Apr 6, 2003
 */
package org.apache.ojb.taglib.broker.support;

import javax.servlet.jsp.JspException;

/**
 * Description
 * 
 * @author <a href="mailto:jacob.hookom@mckesson.com">Jacob Hookom</a>
 * @version ##build.version## - Created: Apr 6, 2003 10:41:57 PM
 */
public abstract class OperatorSupport extends ConditionalSupport
{
	// ***********************************************************
	// Internal State

	protected String property; 	// property the condition is on
	protected Object value; 	// the value to check against
	protected boolean needBody; // need to process body for value

	// ***********************************************************
	// Construction and Initialization

	/**
	 * Creates and initializes the tag.  Because of the nature of
	 * tags, we cannot expect any other constructor to be called
	 * other than the default constructor.
	 */
	public OperatorSupport()
	{
		super();
		this.init();
	}

	/**
	 * Resets the tag's state
	 */
	private void init()
	{
		this.property = null;
		this.value = null;
		this.needBody = false;
	}

	/**
	 * Re-initializes the tag for pooling support by the web container
	 * @see javax.servlet.jsp.tagext.Tag#release()
	 */
	public void release()
	{
		super.release();
		this.init();
	}

	// ***********************************************************
	// Tag Methods

	/**
	 * Evaluates value if need be and calls the process
	 * @see javax.servlet.jsp.tagext.Tag#doStartTag()
	 */
	public int doStartTag() throws JspException
	{
		if (this.value != null)
		{
			this.doProcess();
			return SKIP_BODY;
		}
		else
		{
			this.needBody = true;
			return EVAL_BODY_BUFFERED;
		}
	}

	/**
	 * If the body needed to be read in, call the process now
	 * @see javax.servlet.jsp.tagext.Tag#doEndTag()
	 */
	public int doEndTag() throws JspException
	{
		if (this.needBody)
		{
			if (this.bodyContent != null
				&& this.bodyContent.getString() != null)
			{
				this.value = this.bodyContent.getString().trim();
				this.doProcess();
			}
		}
		return EVAL_PAGE;
	}
}
