/*
 * QuerySupport.java
 * Created on Apr 7, 2003
 */
package org.apache.ojb.taglib.broker.support;

import java.util.Collection;

import javax.servlet.jsp.JspException;

import org.apache.ojb.broker.PersistenceBroker;
import org.apache.ojb.broker.PersistenceBrokerFactory;
import org.apache.ojb.broker.query.Criteria;
import org.apache.ojb.broker.query.QueryByCriteria;
import org.apache.ojb.taglib.resources.TagResources;

/**
 * Description
 * 
 * @author <a href="mailto:jacob.hookom@mckesson.com">Jacob Hookom</a>
 * @version ##build.version## - Created: Apr 7, 2003 1:58:43 AM
 */
public abstract class QuerySupport extends CriteriaSupport
{
	// ***********************************************************
	// Internal State

	protected int start; // start at index
	protected int end; // end at index
	protected boolean distinct; // if it is distinct
	protected QueryByCriteria query; // the criteria object
	protected Object example; // example object to populate criteria
	protected Class type; // the type of object to return
	protected PersistenceBroker broker; // broker to use
	protected boolean localBroker; // if this broker is local

	// ***********************************************************
	// Construction and Initialization

	/**
	 * Creates and initializes the tag.  Because of the nature of
	 * tags, we cannot expect any other constructor to be called
	 * other than the default constructor.
	 */
	public QuerySupport()
	{
		super();
		this.init();
	}

	/**
	 * Resets the tag's state
	 */
	private void init()
	{
		this.query = null;
		this.start = 0;
		this.end = 0;
		this.distinct = true;
		this.example = null;
		this.type = null;
		this.broker = null;
		this.localBroker = false;
	}

	/**
	 * Re-initializes the tag for pooling support by the web container
	 * @see javax.servlet.jsp.tagext.Tag#release()
	 */
	public void release()
	{
		super.release();
		this.init();
	}

	// ***********************************************************
	// Accessor Methods

	public Criteria getCriteria()
	{
		return this.query.getCriteria();
	}
	
	public QueryByCriteria getQuery()
	{
		return this.query;
	}

	// ***********************************************************
	// Tag Methods

	/**
	 * Evaluates value if need be and calls the process
	 * @see javax.servlet.jsp.tagext.Tag#doStartTag()
	 */
	public int doStartTag() throws JspException
	{
		// create query
		this.initQuery();
		
		this.query.setEndAtIndex(this.end);
		this.query.setStartAtIndex(this.start);
		this.query.setDistinct(this.distinct);

		return EVAL_BODY_BUFFERED;
	}

	/**
	 * If the body needed to be read in, call the process now
	 * @see javax.servlet.jsp.tagext.Tag#doEndTag()
	 */
	public int doEndTag() throws JspException
	{
		if (this.broker == null)
		{
			this.broker = PersistenceBrokerFactory.defaultPersistenceBroker();
			this.broker.beginTransaction();
			this.localBroker = true;
		}

		Object result = null;
		try
		{
			result = this.doProcess();
			if (result instanceof Collection)
			{
				System.out.println("result size " + ((Collection) result).size());
			}
		}
		catch (Exception e)
		{
			this.broker.abortTransaction();
			throw new JspException(
				TagResources.getMessage(
					"QUERY_PB_ERROR",
					this.getClass().getName()),
				e);
		}
		finally
		{
			if (this.localBroker)
			{
				if (this.broker.isInTransaction())
					this.broker.commitTransaction();
					
				this.broker.close();
			}
		}

		if (this.var != null)
		{
			this.pageContext.setAttribute(this.var, result, this.scope);
		}
		return EVAL_PAGE;
	}

	// ***********************************************************
	// Supporting Methods

	protected abstract Object doProcess() throws Exception;

	protected void initQuery() throws JspException
	{
		// if the example was provided
		if (this.example != null)
		{
			this.query = new QueryByCriteria(this.example);

			// if additional criteria was also specified, then AND it on
			if (this.criteria != null)
			{
				this.query.getCriteria().addAndCriteria(this.criteria);
			}
		}
		// else we have to find out a type
		else if (this.type != null)
		{
			// if the criteria was null, specify it now
			if (this.criteria == null)
			{
				this.criteria = new Criteria();
			}

			// actually create the criteria
			this.query = new QueryByCriteria(this.type, this.criteria);
		}
		else
		{
			throw new JspException(TagResources.getMessage("QUERY_NO_TYPE"));
		}
	}
}
