/*
 * UpdateSupport.java
 * Created on Apr 7, 2003
 */
package org.apache.ojb.taglib.broker.support;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.BodyTagSupport;

import org.apache.ojb.broker.PersistenceBroker;
import org.apache.ojb.broker.PersistenceBrokerFactory;
import org.apache.ojb.taglib.resources.TagResources;

/**
 * Description
 * 
 * @author <a href="mailto:jacob.hookom@mckesson.com">Jacob Hookom</a>
 * @version ##build.version## - Created: Apr 7, 2003 3:17:38 PM
 */
public abstract class UpdateSupport extends BodyTagSupport
{
	//	***********************************************************
	// Internal State
	
	protected Object value; // example object to populate criteria
	protected PersistenceBroker broker; // broker to use
	protected boolean localBroker; // if this broker is local

	// ***********************************************************
	// Construction and Initialization

	/**
	 * Creates and initializes the tag.  Because of the nature of
	 * tags, we cannot expect any other constructor to be called
	 * other than the default constructor.
	 */
	public UpdateSupport()
	{
		super();
		this.init();
	}

	/**
	 * Resets the tag's state
	 */
	private void init()
	{
		this.value = null;
		this.broker = null;
		this.localBroker = false;
	}

	/**
	 * Re-initializes the tag for pooling support by the web container
	 * @see javax.servlet.jsp.tagext.Tag#release()
	 */
	public void release()
	{
		super.release();
		this.init();
	}

	// ***********************************************************
	// Accessor Methods

	// ***********************************************************
	// Tag Methods

	/**
	 * Evaluates value if need be and calls the process
	 * @see javax.servlet.jsp.tagext.Tag#doStartTag()
	 */
	public int doStartTag() throws JspException
	{
		return SKIP_BODY;
	}

	/**
	 * If the body needed to be read in, call the process now
	 * @see javax.servlet.jsp.tagext.Tag#doEndTag()
	 */
	public int doEndTag() throws JspException
	{
		if (this.broker == null)
		{
			this.broker = PersistenceBrokerFactory.defaultPersistenceBroker();
			this.broker.beginTransaction();
			this.localBroker = true;
		}

		try
		{
			this.doProcess();
		}
		catch (Exception e)
		{
			this.broker.abortTransaction();
			throw new JspException(
				TagResources.getMessage(
					"QUERY_PB_ERROR",
					this.getClass().getName()),
				e);
		}
		finally
		{
			if (this.localBroker)
			{
				if (this.broker.isInTransaction())
					this.broker.commitTransaction();

				this.broker.close();
			}
		}

		return EVAL_PAGE;
	}

	// ***********************************************************
	// Supporting Methods

	protected abstract void doProcess() throws Exception;
}
