/*
 * AbstractFieldOperatorTag.java
 * Created on Apr 7, 2003
 */
package org.apache.ojb.taglib.broker.tags;

import javax.servlet.jsp.JspException;

import org.apache.ojb.taglib.broker.support.FieldOperatorSupport;
import org.apache.taglibs.standard.tag.el.core.ExpressionUtil;

/**
 * Abstract Tag for doing comparisons between fields/values
 * 
 * @author <a href="mailto:jacob.hookom@mckesson.com">Jacob Hookom</a>
 * @version ##build.version## - Created: Apr 7, 2003 1:18:34 AM
 */
public abstract class AbstractFieldOperatorTag extends FieldOperatorSupport
{
	//	***********************************************************
	// Internal State

	private String _field;
	private String _property;
	protected String _tagName;

	// ***********************************************************
	// Constructor and Initialization

	/**
	 * Default Constructor for the AbstractOperatorTag
	 */
	public AbstractFieldOperatorTag()
	{
		super();
		this.init();
	}

	/**
	 * Resets the tag's state
	 */
	protected void init()
	{
		this._field = null;
		this._property = null;
		this._tagName = "field-operator";
	}

	/**
	 * Re-initializes the tag for pooling support by the web container
	 * @see javax.servlet.jsp.tagext.Tag#release()
	 */
	public void release()
	{
		super.release();
		this.init();
	}

	// ***********************************************************
	// Accessors

	public void setField(String field)
	{
		this._field = field;
	}

	public void setProperty(String property)
	{
		this._property = property;
	}

	// ***********************************************************
	// Tag Methods

	/**
	 * @see javax.servlet.jsp.tagext.Tag#doStartTag()
	 */
	public int doStartTag() throws JspException
	{
		this.evaluateExpressions();
		return super.doStartTag();
	}

	// ***********************************************************
	// Support Methods

	/**
	 * Evaluates the values
	 * @throws JspException
	 */
	protected void evaluateExpressions() throws JspException
	{
		this.field =
			ExpressionUtil.evalNotNull(
				this._tagName,
				"field",
				this._field,
				Object.class,
				this,
				this.pageContext);

		this.property =
			(String) ExpressionUtil.evalNotNull(
				this._tagName,
				"property",
				this._property,
				Object.class,
				this,
				this.pageContext);
	}

}
