/*
 * AbstractOperatorTag.java
 * Created on Apr 6, 2003
 */
package org.apache.ojb.taglib.broker.tags;

import javax.servlet.jsp.JspException;

import org.apache.ojb.taglib.broker.support.OperatorSupport;
import org.apache.taglibs.standard.tag.el.core.ExpressionUtil;

/**
 * Base Operator Tag
 * 
 * @author <a href="mailto:jacob.hookom@mckesson.com">Jacob Hookom</a>
 * @version ##build.version## - Created: Apr 6, 2003 11:35:31 PM
 */
public abstract class AbstractOperatorTag extends OperatorSupport
{
	// ***********************************************************
	// Internal State

	private String _value;
	private String _property;
	protected String _tagName;

	// ***********************************************************
	// Constructor and Initialization

	/**
	 * Default Constructor for the AbstractOperatorTag
	 */
	public AbstractOperatorTag()
	{
		super();
		this.init();
	}

	/**
	 * Resets the tag's state
	 */
	protected void init()
	{
		this._value = null;
		this._property = null;
		this._tagName = "operator";
	}

	/**
	 * Re-initializes the tag for pooling support by the web container
	 * @see javax.servlet.jsp.tagext.Tag#release()
	 */
	public void release()
	{
		super.release();
		this.init();
	}

	// ***********************************************************
	// Accessors

	public void setValue(String value)
	{
		this._value = value;
	}

	public void setProperty(String property)
	{
		this._property = property;
	}

	// ***********************************************************
	// Tag Methods

	/**
	 * @see javax.servlet.jsp.tagext.Tag#doStartTag()
	 */
	public int doStartTag() throws JspException
	{
		this.evaluateExpressions();
		return super.doStartTag();
	}

	// ***********************************************************
	// Support Methods

	/**
	 * Evaluates the values
	 * @throws JspException
	 */
	protected void evaluateExpressions() throws JspException
	{
		this.value =
				ExpressionUtil.evalNotNull(
					this._tagName,
					"value",
					this._value,
					Object.class,
					this,
					this.pageContext);

		this.property =
			(String) ExpressionUtil.evalNotNull(
				this._tagName,
				"property",
				this._property,
				Object.class,
				this,
				this.pageContext);
	}

}
