/*
 * AbstractQueryTag.java
 * Created on Apr 7, 2003
 */
package org.apache.ojb.taglib.broker.tags;

import javax.servlet.jsp.JspException;

import org.apache.ojb.broker.PersistenceBroker;
import org.apache.ojb.taglib.resources.TagResources;
import org.apache.ojb.taglib.broker.support.QuerySupport;
import org.apache.taglibs.standard.tag.common.core.NullAttributeException;
import org.apache.taglibs.standard.tag.common.core.Util;
import org.apache.taglibs.standard.tag.el.core.ExpressionUtil;

/**
 * Base Tag to be used in defining Object for querying via broker
 * 
 * @author <a href="mailto:jacob.hookom@mckesson.com">Jacob Hookom</a>
 * @version ##build.version## - Created: Apr 7, 2003 2:55:40 AM
 */
public abstract class AbstractQueryTag extends QuerySupport
{
	//	***********************************************************
	// Internal State

	protected String _start; // start at index
	protected String _end; // end at index
	protected String _distinct; // if it is distinct
	protected String _example; // example object to populate criteria
	protected String _type; // the type of object to return
	protected String _broker; // broker to use
	protected String _var; // var to store result(s) in
	protected String _scope; // scope to store results in
	protected String _tagName;

	// ***********************************************************
	// Constructor and Initialization

	/**
	 * Default Constructor for the AbstractOperatorTag
	 */
	public AbstractQueryTag()
	{
		super();
		this.init();
	}

	/**
	 * Resets the tag's state
	 */
	protected void init()
	{
		this._start = null;
		this._end = null;
		this._distinct = null;
		this._example = null;
		this._type = null;
		this._broker = null;
		this._var = null;
		this._scope = null;
		this._tagName = "query";
	}

	/**
	 * Re-initializes the tag for pooling support by the web container
	 * @see javax.servlet.jsp.tagext.Tag#release()
	 */
	public void release()
	{
		super.release();
		this.init();
	}

	// ***********************************************************
	// Accessors
	
	public void setVar(String var)
	{
		this._var = var;
	}
	
	public void setScope(String scope)
	{
		this._scope = scope;
	}

	public void setStart(String start)
	{
		this._start = start;
	}

	public void setEnd(String end)
	{
		this._end = end;
	}

	public void setDistinct(String distinct)
	{
		this._distinct = distinct;
	}

	public void setExample(String example)
	{
		this._example = example;
	}

	public void setType(String type)
	{
		this._type = type;
	}

	public void setBroker(String broker)
	{
		this._broker = broker;
	}

	// ***********************************************************
	// Tag Methods

	/**
	 * @see javax.servlet.jsp.tagext.Tag#doStartTag()
	 */
	public int doStartTag() throws JspException
	{
		this.evaluateExpressions();
		return super.doStartTag();
	}

	// ***********************************************************
	// Support Methods

	/**
	 * Evaluates the values
	 * @throws JspException
	 */
	protected void evaluateExpressions() throws JspException
	{
		Object temp = null;

		// START INDEX
		if (this._start != null)
		{
			try
			{
				temp =
					ExpressionUtil.evalNotNull(
						this._tagName,
						"start",
						this._start,
						Object.class,
						this,
						this.pageContext);

				this.start = Integer.parseInt(temp.toString());
			}
			catch (NullAttributeException nae)
			{
				// do nothing
			}
		}

		// END INDEX
		if (this._end != null)
		{
			try
			{
				temp =
					ExpressionUtil.evalNotNull(
						this._tagName,
						"end",
						this._end,
						Object.class,
						this,
						this.pageContext);

				this.end = Integer.parseInt(temp.toString());
			}
			catch (NullAttributeException nae)
			{
				// do nothing
			}
		}

		// DISTINCT
		if (this._distinct != null)
		{
			if (this._distinct.equalsIgnoreCase("false"))
				this.distinct = false;
		}

		// EXAMPLE OBJECT
		try
		{
			this.example =
				ExpressionUtil.evalNotNull(
					this._tagName,
					"example",
					this._example,
					Object.class,
					this,
					this.pageContext);
		}
		catch (NullAttributeException nae)
		{
			this.example = null;
		}

		// TYPE SPECIFICATION
		if (this._type != null)
		{
			try
			{
				this.type = Class.forName(this._type);
			}
			catch (ClassNotFoundException cnfe)
			{
				throw new JspException(
					TagResources.getMessage("QUERY_BAD_TYPE", this._type));
			}
		}

		// GET BROKER
		if (this._broker != null)
		{
			try
			{
				temp =
					ExpressionUtil.evalNotNull(
						this._tagName,
						"broker",
						this._broker,
						Object.class,
						this,
						this.pageContext);

				if (temp instanceof PersistenceBroker)
				{
					this.broker = (PersistenceBroker) temp;
				}
				else
				{
					throw new JspException(
						TagResources.getMessage("QUERY_BAD_PB", this._broker));
				}
			}
			catch (NullAttributeException nae)
			{
				this.broker = null;
			}
		}

		// VAR TO STORE IT
		this.var = this._var;

		// SCOPE
		this.scope = Util.getScope(this._scope);
	}
}
