/*
 * OrderByTag.java
 * Created on Apr 7, 2003
 */
package org.apache.ojb.taglib.broker.tags;

import javax.servlet.jsp.JspException;

import org.apache.ojb.broker.query.QueryByCriteria;
import org.apache.ojb.taglib.broker.support.OrderBySupport;
import org.apache.taglibs.standard.tag.common.core.NullAttributeException;
import org.apache.taglibs.standard.tag.el.core.ExpressionUtil;

/**
 * Nested in a query tag, or a criteria-type tag to add criteria
 * 
 * @author <a href="mailto:jacob.hookom@mckesson.com">Jacob Hookom</a>
 * @version ##build.version## - Created: Apr 7, 2003 10:14:15 AM
 */
public final class OrderByTag extends OrderBySupport
{
	//	***********************************************************
	// Internal State

	private String _sort;
	private String _property;

	// ***********************************************************
	// Constructor and Initialization

	/**
	 * Default Constructor for the AbstractOperatorTag
	 */
	public OrderByTag()
	{
		super();
		this.init();
	}

	/**
	 * Resets the tag's state
	 */
	protected void init()
	{
		this._sort = null;
		this._property = null;
	}

	/**
	 * Re-initializes the tag for pooling support by the web container
	 * @see javax.servlet.jsp.tagext.Tag#release()
	 */
	public void release()
	{
		super.release();
		this.init();
	}

	// ***********************************************************
	// Accessors

	public void setSort(String sort)
	{
		this._sort = sort;
	}

	public void setProperty(String property)
	{
		this._property = property;
	}

	// ***********************************************************
	// Tag Methods

	/**
	 * @see javax.servlet.jsp.tagext.Tag#doStartTag()
	 */
	public int doStartTag() throws JspException
	{
		this.evaluateExpressions();
		return super.doStartTag();
	}

	// ***********************************************************
	// Support Methods

	/**
	 * Evaluates the values
	 * @throws JspException
	 */
	protected void evaluateExpressions() throws JspException
	{
		if (this._sort != null)
		{
			try
			{
				this._sort =
					(String) ExpressionUtil.evalNotNull(
						"orderBy",
						"sort",
						this._sort,
						Object.class,
						this,
						this.pageContext);
						
				if (this._sort.equalsIgnoreCase("desc"))
					this.sortAsc = false;
				else
					this.sortAsc = true;
			}
			catch (NullAttributeException nae)
			{
				this.sortAsc = true;
			}
		}

		this.property =
			(String) ExpressionUtil.evalNotNull(
				"orderBy",
				"property",
				this._property,
				Object.class,
				this,
				this.pageContext);
	}

	/* (non-Javadoc)
	 * @see org.apache.ojb.taglib.broker.support.op.ConditionalSupport#doProcess()
	 */
	protected void doProcess() throws JspException
	{
		QueryByCriteria query = this.getQuery();
		query.addOrderBy(this.property, this.sortAsc);
	}

}
